import { LanguageService, getRange, getTokenAtPosition, } from 'graphql-language-service';
import { coreDefinitionSource, coreDiagnosticsSource, coreHoverSource, toGraphQLPosition, toMonacoRange, removeFalsey, } from './utils';
export class EnrichedLanguageService extends LanguageService {
    async getNodeAtPosition(schema, document, position) {
        if (schema) {
            const token = getTokenAtPosition(document, position);
            if (token) {
                return token;
            }
        }
        return null;
    }
    async buildBridgeForProviders(model, position) {
        const graphQLPosition = toGraphQLPosition(position);
        const document = model.getValue();
        const schema = await this.getSchema();
        if (!schema) {
            return null;
        }
        const tokenAtPosition = await this.getNodeAtPosition(schema, document, graphQLPosition);
        if (!tokenAtPosition) {
            return null;
        }
        return {
            languageService: this,
            position: graphQLPosition,
            document,
            schema,
            token: tokenAtPosition,
            model,
        };
    }
    getDefinitionProvider(rawSources) {
        const sources = [...rawSources, coreDefinitionSource];
        return {
            provideDefinition: async (model, position) => {
                const bridge = await this.buildBridgeForProviders(model, position);
                if (!bridge) {
                    return [];
                }
                const nestedArrays = (await Promise.all(sources.map(source => source.forNode(bridge)))).filter(Boolean);
                const items = [].concat(...nestedArrays);
                return items;
            },
        };
    }
    getHoverProvider(rawSources) {
        const sources = [...rawSources, coreHoverSource];
        return {
            provideHover: async (model, position) => {
                const info = await this.buildBridgeForProviders(model, position);
                if (!info) {
                    return;
                }
                const contents = await Promise.all(sources.map(async (source) => {
                    try {
                        return await source.forNode(info);
                    }
                    catch (e) {
                        return null;
                    }
                }));
                return {
                    contents: contents.filter(removeFalsey),
                    range: toMonacoRange(getRange({
                        column: info.position.character,
                        line: info.position.line + 1,
                    }, info.document)),
                };
            },
        };
    }
    async handleDecorations(decorationSources, model, monacoInstance, editorInstance) {
        for (const source of decorationSources) {
            source.forDocument({
                monaco: monacoInstance,
                editor: editorInstance,
                document: model.getValue(),
                model,
                languageService: this,
            });
        }
    }
    async handleDiagnostics(rawDiagnosticsSources, model, monacoInstance) {
        const diagnosticsSources = [...rawDiagnosticsSources, coreDiagnosticsSource];
        const nestedArrays = (await Promise.all(diagnosticsSources.map(async (source) => {
            try {
                return await source.forDocument({
                    languageService: this,
                    model,
                    document: model.getValue().toString(),
                });
            }
            catch (e) {
                return null;
            }
        }))).filter(removeFalsey);
        const markerData = [].concat(...nestedArrays);
        monacoInstance.editor.setModelMarkers(model, 'graphql', markerData);
    }
    getModelChangeHandler() {
        return async (editorInstance, monacoInstance, diagnosticsSources, decorationsSources) => {
            const model = editorInstance.getModel();
            if (!model) {
                return;
            }
            await Promise.all([
                this.handleDiagnostics(diagnosticsSources, model, monacoInstance),
                this.handleDecorations(decorationsSources, model, monacoInstance, editorInstance),
            ]);
        };
    }
    trySchema(sdl) {
        // XX-DOTAN: In case of an issue with the schema parsing, we'll get the error through
        // the Marker, and we can safely ignore it here.
        return this.setSchema(sdl).then(() => this.getSchema());
    }
}
