import { jsx as _jsx } from "react/jsx-runtime";
import { useEffect, useRef, useState } from 'react';
import MonacoEditor, { useMonaco } from '@monaco-editor/react';
import { getAutocompleteSuggestions,
// CompletionItemKind as lsCIK,
 } from 'graphql-language-service';
import * as languages from './enums';
import { toGraphQLPosition, toMonacoRange } from './utils';
import { useTheme } from 'next-themes';
// This enum `CompletionItemKind` exist on graphql-language-service v4
var lsCIK;
(function (lsCIK) {
    lsCIK[lsCIK["Text"] = 1] = "Text";
    lsCIK[lsCIK["Method"] = 2] = "Method";
    lsCIK[lsCIK["Function"] = 3] = "Function";
    lsCIK[lsCIK["Constructor"] = 4] = "Constructor";
    lsCIK[lsCIK["Field"] = 5] = "Field";
    lsCIK[lsCIK["Variable"] = 6] = "Variable";
    lsCIK[lsCIK["Class"] = 7] = "Class";
    lsCIK[lsCIK["Interface"] = 8] = "Interface";
    lsCIK[lsCIK["Module"] = 9] = "Module";
    lsCIK[lsCIK["Property"] = 10] = "Property";
    lsCIK[lsCIK["Unit"] = 11] = "Unit";
    lsCIK[lsCIK["Value"] = 12] = "Value";
    lsCIK[lsCIK["Enum"] = 13] = "Enum";
    lsCIK[lsCIK["Keyword"] = 14] = "Keyword";
    lsCIK[lsCIK["Snippet"] = 15] = "Snippet";
    lsCIK[lsCIK["Color"] = 16] = "Color";
    lsCIK[lsCIK["File"] = 17] = "File";
    lsCIK[lsCIK["Reference"] = 18] = "Reference";
    lsCIK[lsCIK["Folder"] = 19] = "Folder";
    lsCIK[lsCIK["EnumMember"] = 20] = "EnumMember";
    lsCIK[lsCIK["Constant"] = 21] = "Constant";
    lsCIK[lsCIK["Struct"] = 22] = "Struct";
    lsCIK[lsCIK["Event"] = 23] = "Event";
    lsCIK[lsCIK["Operator"] = 24] = "Operator";
    lsCIK[lsCIK["TypeParameter"] = 25] = "TypeParameter";
})(lsCIK || (lsCIK = {}));
const toCompletionItemKind = (kind) => {
    const CIK = languages.CompletionItemKind;
    const map = {
        [lsCIK.Text]: CIK.Text,
        [lsCIK.Method]: CIK.Method,
        [lsCIK.Function]: CIK.Function,
        [lsCIK.Constructor]: CIK.Constructor,
        [lsCIK.Field]: CIK.Field,
        [lsCIK.Variable]: CIK.Variable,
        [lsCIK.Class]: CIK.Class,
        [lsCIK.Interface]: CIK.Interface,
        [lsCIK.Module]: CIK.Module,
        [lsCIK.Property]: CIK.Property,
        [lsCIK.Unit]: CIK.Unit,
        [lsCIK.Value]: CIK.Value,
        [lsCIK.Enum]: CIK.Enum,
        [lsCIK.Keyword]: CIK.Keyword,
        [lsCIK.Snippet]: CIK.Snippet,
        [lsCIK.Color]: CIK.Color,
        [lsCIK.File]: CIK.File,
        [lsCIK.Reference]: CIK.Reference,
        [lsCIK.Folder]: CIK.Folder,
        [lsCIK.EnumMember]: CIK.EnumMember,
        [lsCIK.Constant]: CIK.Constant,
        [lsCIK.Struct]: CIK.Struct,
        [lsCIK.Event]: CIK.Event,
        [lsCIK.Operator]: CIK.Operator,
        [lsCIK.TypeParameter]: CIK.TypeParameter,
    };
    return map[kind] || CIK.Text;
};
const toCompletion = (entry) => {
    var _a;
    return {
        range: entry.range,
        kind: toCompletionItemKind(entry.kind),
        label: entry.label,
        insertText: (_a = entry.insertText) !== null && _a !== void 0 ? _a : entry.label,
        insertTextRules: entry.insertText ? languages.CompletionItemInsertTextRule.InsertAsSnippet : undefined,
        sortText: entry.sortText,
        filterText: entry.filterText,
        documentation: entry.documentation,
        detail: entry.detail,
        command: entry.command,
    };
};
class GraphQLWorker {
    doComplete(documentModel, position, schema) {
        const document = documentModel.getValue();
        if (!document) {
            console.log('no document');
            return [];
        }
        if (!schema) {
            console.log('no schema');
        }
        const graphQLPosition = toGraphQLPosition(position);
        const suggestions = getAutocompleteSuggestions(schema, document, graphQLPosition);
        return suggestions.map(suggestion => this.toCompletion(suggestion));
    }
    toCompletion(entry, range) {
        return {
            label: entry.label,
            insertText: entry.insertText,
            insertTextFormat: entry.insertTextFormat,
            sortText: entry.sortText,
            filterText: entry.filterText,
            documentation: entry.documentation,
            detail: entry.detail,
            range: range ? toMonacoRange(range) : undefined,
            kind: entry.kind,
            command: entry.command ? { ...entry.command, id: entry.command.command } : undefined,
        };
    }
}
export const ExecutableDocumentEditor = ({ schema, ...editorProps }) => {
    const { theme } = useTheme();
    const monaco = useMonaco();
    const [completionProvider, setCompletionProvider] = useState(null);
    const editorUriRef = useRef();
    useEffect(() => {
        if (!monaco || !schema) {
            return;
        }
        if (completionProvider) {
            // dispose instance to prevent having duplicated field
            completionProvider.dispose();
        }
        const newProvider = monaco.languages.registerCompletionItemProvider('graphql', {
            triggerCharacters: [':', '$', '\n', ' ', '(', '@'],
            provideCompletionItems(model, position) {
                const isUriEquals = model.uri.path === editorUriRef.current.path;
                if (!isUriEquals) {
                    return { suggestions: [] };
                }
                const worker = new GraphQLWorker();
                const completionItems = worker.doComplete(model, position, schema);
                return {
                    incomplete: true,
                    suggestions: completionItems.map(item => toCompletion(item)),
                };
            },
        });
        setCompletionProvider(newProvider);
    }, [monaco, schema]);
    return (_jsx(MonacoEditor, { height: "70vh", theme: theme === 'dark' ? 'vs-dark' : 'light', ...editorProps, language: "graphql", onMount: editor => {
            editorUriRef.current = editor.getModel().uri;
        } }));
};
