import { useState, useMemo, useEffect } from 'react';
import { EnrichedLanguageService } from './enriched-language-service';
import { isInterfaceType, isObjectType } from 'graphql';
import { emptyLocation, locToRange } from './utils';
export const useSchemaServices = (options = {}) => {
    const [editorRef, setEditor] = useState(null);
    const [monacoRef, setMonaco] = useState(null);
    const languageService = useMemo(() => options.sharedLanguageService ||
        new EnrichedLanguageService({
            schemaString: options.schema,
            schemaConfig: {
                buildSchemaOptions: {
                    assumeValid: true,
                    assumeValidSDL: true,
                },
            },
        }), [options.sharedLanguageService]);
    useEffect(() => {
        if (monacoRef && editorRef) {
            if (options.keyboardShortcuts) {
                for (const action of options.keyboardShortcuts(editorRef, monacoRef)) {
                    editorRef.addAction(action);
                }
            }
            for (const action of options.actions || []) {
                editorRef.addAction({
                    id: action.id,
                    label: action.label,
                    keybindings: action.keybindings,
                    contextMenuGroupId: action.contextMenuGroupId || 'navigation',
                    contextMenuOrder: action.contextMenuOrder,
                    async run(editor) {
                        const model = editor.getModel();
                        const position = editor.getPosition();
                        if (model && position) {
                            const bridge = await languageService.buildBridgeForProviders(model, position);
                            if (bridge) {
                                action.onRun({ editor: editorRef, monaco: monacoRef, bridge });
                            }
                        }
                    },
                });
            }
            const handler = languageService.getModelChangeHandler();
            handler(editorRef, monacoRef, options.diagnosticsProviders || [], options.decorationsProviders || []);
            const onChangeDisposable = editorRef.onDidChangeModelContent(() => handler(editorRef, monacoRef, options.diagnosticsProviders || [], options.decorationsProviders || []));
            const definitionProviderDisposable = monacoRef.languages.registerDefinitionProvider('graphql', languageService.getDefinitionProvider(options.definitionProviders || []));
            const hoverDisposable = monacoRef.languages.registerHoverProvider('graphql', languageService.getHoverProvider(options.hoverProviders || []));
            return () => {
                hoverDisposable === null || hoverDisposable === void 0 ? void 0 : hoverDisposable.dispose();
                definitionProviderDisposable === null || definitionProviderDisposable === void 0 ? void 0 : definitionProviderDisposable.dispose();
                onChangeDisposable === null || onChangeDisposable === void 0 ? void 0 : onChangeDisposable.dispose();
            };
        }
        // eslint-disable-next-line @typescript-eslint/no-empty-function
        return () => { };
    }, [editorRef, monacoRef]);
    return {
        setEditor,
        setMonaco,
        editorRef,
        monacoRef,
        languageService,
        setSchema: (newValue) => languageService.trySchema(newValue),
        editorApi: {
            jumpToType: (typeName) => {
                languageService.getSchema().then(schema => {
                    var _a;
                    if (schema) {
                        const type = schema.getType(typeName);
                        if ((_a = type === null || type === void 0 ? void 0 : type.astNode) === null || _a === void 0 ? void 0 : _a.loc) {
                            const range = locToRange(type.astNode.loc);
                            editorRef === null || editorRef === void 0 ? void 0 : editorRef.setSelection(range);
                            editorRef === null || editorRef === void 0 ? void 0 : editorRef.revealPositionInCenter({ column: 0, lineNumber: range.startLineNumber }, 0);
                        }
                    }
                });
            },
            jumpToField(typeName, fieldName) {
                languageService.getSchema().then(schema => {
                    var _a;
                    if (schema) {
                        const type = schema.getType(typeName);
                        if (type && (isObjectType(type) || isInterfaceType(type))) {
                            const field = type.getFields()[fieldName];
                            if ((_a = field === null || field === void 0 ? void 0 : field.astNode) === null || _a === void 0 ? void 0 : _a.loc) {
                                const range = locToRange(field.astNode.loc);
                                editorRef === null || editorRef === void 0 ? void 0 : editorRef.setSelection(range);
                                editorRef === null || editorRef === void 0 ? void 0 : editorRef.revealPositionInCenter({ column: 0, lineNumber: range.startLineNumber }, 0);
                            }
                        }
                    }
                });
            },
            deselect: () => editorRef === null || editorRef === void 0 ? void 0 : editorRef.setSelection(emptyLocation),
        },
    };
};
