import { Position, getHoverInformation, getRange, DIAGNOSTIC_SEVERITY, } from 'graphql-language-service';
export { getRange };
export function removeFalsey(obj) {
    return Boolean(obj);
}
export function locToRange(loc) {
    return {
        startLineNumber: loc.startToken.line,
        startColumn: loc.startToken.column,
        endLineNumber: loc.endToken.line + 1,
        endColumn: loc.endToken.column,
    };
}
export const emptyLocation = {
    startLineNumber: 0,
    startColumn: 0,
    endLineNumber: 0,
    endColumn: 0,
};
export const coreDiagnosticsSource = {
    async forDocument({ model, document, languageService }) {
        return languageService
            .getDiagnostics(model.uri.toString(), document)
            .then(diag => diag.map(toMarkerData))
            .catch(e => {
            if ('message' in e && 'locations' in e) {
                return [
                    toMarkerData({
                        severity: DIAGNOSTIC_SEVERITY.Error,
                        message: e.message,
                        source: 'GraphQL: Syntax',
                        range: getRange(e.locations[0], document),
                    }),
                ];
            }
            else {
                console.warn(`GraphQL getDiagnostics failed unexpected error: `, e);
                return [];
            }
        });
    },
};
export const coreDefinitionSource = {
    forNode: ({ schema, model, token }) => {
        if (token.state && token.state.kind === 'NamedType' && token.state.name) {
            const type = schema.getType(token.state.name);
            if (type && type.astNode && type.astNode.loc) {
                return [
                    {
                        range: {
                            startLineNumber: type.astNode.loc.startToken.line,
                            startColumn: type.astNode.loc.startToken.column,
                            endLineNumber: type.astNode.loc.endToken.line + 1,
                            endColumn: type.astNode.loc.endToken.column,
                        },
                        uri: model.uri,
                    },
                ];
            }
        }
        return [];
    },
};
export const coreHoverSource = {
    forNode: ({ schema, document, position, token }) => ({
        value: getHoverInformation(schema, document, position, token),
    }),
};
export const debugHoverSource = {
    forNode: ({ token }) => ({
        value: `\`\`\`json\n${JSON.stringify(token.state, null, 2)}\n\`\`\``,
    }),
};
export function toGraphQLPosition(position) {
    return new Position(position.lineNumber - 1, position.column - 1);
}
export function toMarkerData(diagnostic) {
    return {
        startLineNumber: diagnostic.range.start.line + 1,
        endLineNumber: diagnostic.range.end.line + 1,
        startColumn: diagnostic.range.start.character + 1,
        endColumn: diagnostic.range.end.character,
        message: diagnostic.message,
        severity: 5,
        // severity: toMonacoSeverity(diagnostic.severity),
        code: diagnostic.code || undefined,
    };
}
export function toMonacoRange(range) {
    return {
        startLineNumber: range.start.line + 1,
        startColumn: range.start.character + 1,
        endLineNumber: range.end.line + 1,
        endColumn: range.end.character + 1,
    };
}
export function showWidgetInPosition(editorInstance, position, htmlElement) {
    editorInstance.changeViewZones(function (changeAccessor) {
        changeAccessor.addZone({
            afterLineNumber: position.line + 1,
            heightInPx: 60,
            domNode: htmlElement,
        });
    });
}
